import type { Attributes } from 'soukai';
import { BelongsToManyRelation } from 'soukai';
import { BelongsToOneRelation } from 'soukai';
import type { BootedFieldDefinition } from 'soukai';
import type { BootedFieldsDefinition } from 'soukai';
import { Constructor } from '@noeldemartin/utils';
import { Cr } from '@noeldemartin/utils';
import type { Engine } from 'soukai';
import type { EngineDocument } from 'soukai';
import type { EngineDocumentsCollection } from 'soukai';
import type { EngineFilters } from 'soukai';
import type { EngineUpdates } from 'soukai';
import type { Fetch as Fetch_2 } from '@noeldemartin/solid-utils';
import type { FieldDefinition } from 'soukai';
import type { FieldsDefinition } from 'soukai';
import type { GetArrayFields } from 'soukai';
import type { GetFieldsDefinition } from 'soukai';
import { HasManyRelation } from 'soukai';
import { HasOneRelation } from 'soukai';
import type { JsonLD } from '@noeldemartin/solid-utils';
import type { JsonLDGraph } from '@noeldemartin/solid-utils';
import type { Key } from 'soukai';
import type { MagicAttributeProperties } from 'soukai';
import type { MagicAttributes } from 'soukai';
import { Model } from 'soukai';
import type { ModelCastAttributeOptions } from 'soukai';
import type { ModelConstructor } from 'soukai';
import { ModelKey } from 'soukai';
import { MultiModelRelation } from 'soukai';
import type { Pretty } from '@noeldemartin/utils';
import type { Quad } from 'rdf-js';
import type { Relation } from 'soukai';
import type { RelationCloneOptions } from 'soukai';
import type { SchemaDefinition } from 'soukai';
import { SingleModelRelation } from 'soukai';
import { SolidDocumentPermission } from '@noeldemartin/solid-utils';
import type { SolidUserProfile } from '@noeldemartin/solid-utils';
import type { TimestampFieldValue } from 'soukai';

export declare class AddPropertyOperation extends AddPropertyOperationSchema {
    protected applyPropertyUpdate(model: SolidModel, field: string): void;
}

export declare const AddPropertyOperationSchema: Constructor<    {
id: string;
resourceUrl: any;
value: any;
date: Date;
property: any;
createdAt: Date;
updatedAt: Date;
}> & Constructor<PropertyOperation> & typeof SolidModel;

export declare type AnyFetch = (input: any, options?: any) => Promise<any>;

export declare interface BeforeParentCreateRelation extends Relation {
    __beforeParentCreate(): void;
}

export declare function bootSolidModels(): void;

declare const _default: SoukaiSolid;
export default _default;

export declare function defineSolidModelSchema<Schema extends SolidSchemaDefinition>(definition: Schema): Constructor<SolidMagicAttributes<Schema>> & SolidModelConstructor;

export declare function defineSolidModelSchema<BaseModel extends SolidModel, Schema extends SolidSchemaDefinition>(baseModel: SolidModelConstructor<BaseModel>, definition: Schema): Constructor<SolidMagicAttributes<Schema>> & SolidModelConstructor<BaseModel>;

export declare class DeleteOperation extends DeleteOperationSchema {
    apply(model: SolidModel): void;
}

export declare const DeleteOperationSchema: Constructor<    {
id: string;
createdAt: Date;
updatedAt: Date;
}> & Constructor<Operation> & typeof SolidModel;

export declare class DeletesModels {
    protected deleteModels(this: DeletesModelsThis, models: SolidModel[]): Promise<void>;
    private decantDocumentModelsByContainer;
    private decantModelsByDocument;
    private deleteContainerDocumentsModels;
    private deleteDocumentModels;
}

export declare type DeletesModelsThis = SolidModel;

export declare class DocumentContainsManyRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends MultiModelRelation<Parent, Related, RelatedClass> implements DocumentContainsRelation {
    constructor(parent: Parent, relatedClass: RelatedClass);
    setForeignAttributes(): void;
    load(): Promise<Related[]>;
    create(attributes?: Attributes): Promise<Related>;
    __loadDocumentModels(documentUrl: string, document: JsonLDGraph): Promise<void>;
}

export declare interface DocumentContainsRelation {
    __loadDocumentModels(documentUrl: string, document: JsonLDGraph): Promise<void>;
}

export declare type Fetch = TypedFetch | AnyFetch;

export declare function hasBeforeParentCreateHook(relation: Relation): relation is BeforeParentCreateRelation;

export declare const historyModels: {
    AddPropertyOperation: typeof AddPropertyOperation;
    DeleteOperation: typeof DeleteOperation;
    Metadata: typeof Metadata;
    Operation: typeof Operation;
    PropertyOperation: typeof PropertyOperation;
    RemovePropertyOperation: typeof RemovePropertyOperation;
    SetPropertyOperation: typeof SetPropertyOperation;
    Tombstone: typeof Tombstone;
    UnsetPropertyOperation: typeof UnsetPropertyOperation;
};

export declare function isContainer(model: SolidModel): model is SolidContainer;

export declare function isContainerClass(modelClass: SolidModelConstructor): modelClass is SolidContainerConstructor;

export declare interface ISolidDocumentRelation<Related extends SolidModel = SolidModel> extends DocumentContainsRelation {
    reset(related?: Related[]): void;
    __beforeParentCreate(): void;
}

export declare function isSolidBelongsToRelation(relation: unknown): relation is SolidBelongsToRelation;

export declare function isSolidDocumentRelation(relation: Relation): relation is SolidDocumentRelationInstance;

export declare function isSolidHasRelation(relation: unknown): relation is SolidHasRelation;

export declare function isSolidMultiModelDocumentRelation(relation: Relation): relation is SolidMultiModelDocumentRelationInstance;

export declare function isSolidSingleModelDocumentRelation(relation: Relation): relation is SolidSingleModelDocumentRelationInstance;

export declare class ManagesPermissions {
    protected _publicPermissions?: SolidDocumentPermission[];
    get isPublic(): boolean | null;
    get isPrivate(): boolean | null;
    fetchPublicPermissionsIfMissing(this: ManagesPermissionsThis): Promise<void>;
    fetchPublicPermissions(this: ManagesPermissionsThis): Promise<void>;
    updatePublicPermissions(this: ManagesPermissionsThis, permissions: SolidDocumentPermission[]): Promise<void>;
    protected trackPublicPermissions(this: ManagesPermissionsThis): PermissionsTracker;
    private updateAuthorizations;
    private createDocumentACLResource;
    private createPublicAuthorization;
}

export declare type ManagesPermissionsThis = SolidModel;

export declare class Metadata extends MetadataSchema {
    getCreatedAtAttribute(): Date;
    getUpdatedAtAttribute(): Date;
    protected newUrl(documentUrl?: string, resourceHash?: string): string;
}

export declare const MetadataSchema: Constructor<    {
id: string;
resourceUrl: any;
createdAt?: Date | undefined;
updatedAt?: Date | undefined;
deletedAt?: Date | undefined;
}> & Constructor<SolidModel> & typeof SolidModel;

export declare class Operation extends OperationSchema {
    apply(model: SolidModel): void;
    protected newUrl(documentUrl?: string, resourceHash?: string): string;
}

export declare type Operations = {
    Operation: typeof Operation;
    PropertyOperation: typeof PropertyOperation;
    AddPropertyOperation: typeof AddPropertyOperation;
    RemovePropertyOperation: typeof RemovePropertyOperation;
    SetPropertyOperation: typeof SetPropertyOperation;
    UnsetPropertyOperation: typeof UnsetPropertyOperation;
    DeleteOperation: typeof DeleteOperation;
};

export declare const OperationSchema: Constructor<    {
id: string;
resourceUrl: any;
date: Date;
}> & Constructor<SolidModel> & typeof SolidModel;

export declare class OperationsRelation<Parent extends SolidModel = SolidModel> extends SolidHasManyRelation<Parent, Operation, typeof Operation> {
    private static operationMatchers;
    constructor(parent: Parent);
    attach(modelOrAttributes?: Operation | Attributes): Operation;
    attach<T extends keyof Operations>(attributes: Attributes, type: T): InstanceType<Operations[T]>;
    attachAddOperation(attributes: Attributes): AddPropertyOperation;
    attachRemoveOperation(attributes: Attributes): RemovePropertyOperation;
    attachSetOperation(attributes: Attributes): SetPropertyOperation;
    attachUnsetOperation(attributes: Attributes): UnsetPropertyOperation;
    attachDeleteOperation(attributes: Attributes): DeleteOperation;
    __loadDocumentModels(documentUrl: string, document: JsonLDGraph): Promise<void>;
    protected loadRelatedModels(): Promise<Operation[]>;
    protected getOperationClass(resource: JsonLD): typeof Operation | undefined;
    private getOperationMatchers;
}

export declare interface PermissionsTracker {
    documentPermissions: Record<string, SolidDocumentPermission[]>;
    stopTracking(): void;
}

export declare class PropertyOperation extends PropertyOperationSchema {
    apply(model: SolidModel): void;
    protected applyPropertyUpdate(model: SolidModel, field: string): void;
}

export declare const PropertyOperationSchema: Constructor<    {
id: string;
resourceUrl: any;
date: Date;
property: any;
createdAt: Date;
updatedAt: Date;
}> & Constructor<Operation> & typeof SolidModel;

export declare interface ProtectedSolidBelongsToRelation<Related extends SolidModel = SolidModel> {
    loadDocumentModels: SolidDocumentRelation<Related>['loadDocumentModels'];
}

export declare interface ProtectedSolidHasRelation<Related extends SolidModel = SolidModel> {
    loadDocumentModels: SolidDocumentRelation<Related>['loadDocumentModels'];
}

export declare interface ProtectedSolidMultiRelation<Related extends SolidModel = SolidModel> {
    assertLoaded(method: string): this is {
        related: Related[];
    };
}

export declare interface RDFDocumentMetadata {
    containsRelativeIRIs?: boolean;
    describedBy?: string;
    headers?: Headers;
}

export declare class RemovePropertyOperation extends RemovePropertyOperationSchema {
    private static attributeCasters;
    protected applyPropertyUpdate(model: SolidModel, field: string): void;
    private castModelAttribute;
    private createAttributeCaster;
}

export declare const RemovePropertyOperationSchema: Constructor<    {
id: string;
resourceUrl: any;
value: any;
date: Date;
property: any;
createdAt: Date;
updatedAt: Date;
}> & Constructor<PropertyOperation> & typeof SolidModel;

export declare class SerializesToJsonLD {
    protected serializeToJsonLD(this: SerializesToJsonLDThis, includeRelations?: boolean): JsonLD;
    protected parseEngineDocumentAttributesFromJsonLD(this: SerializesToJsonLDThis, document: EngineDocument, resourceId: string): Promise<Attributes>;
    protected convertEngineFiltersToJsonLD(this: SerializesToJsonLDThis, filters: EngineFilters, compactIRIs: boolean): EngineFilters;
    protected convertEngineUpdatesToJsonLD(this: SerializesToJsonLDThis, updates: EngineUpdates, compactIRIs: boolean): EngineUpdates;
    private convertAttributeValuesToJsonLD;
}

export declare type SerializesToJsonLDThis = SolidModel;

export declare class SetPropertyOperation extends SetPropertyOperationSchema {
    protected applyPropertyUpdate(model: SolidModel, field: string): void;
}

export declare const SetPropertyOperationSchema: Constructor<    {
id: string;
resourceUrl: any;
value: any;
date: Date;
property: any;
createdAt: Date;
updatedAt: Date;
}> & Constructor<PropertyOperation> & typeof SolidModel;

export declare class SolidACLAuthorization extends SolidACLAuthorizationSchema {
    /**
     * @deprecated Use requireFetch() instead.
     */
    static get fetch(): Fetch_2;
    static modeFromSolidDocumentPermission(permission: SolidDocumentPermission): string;
}

export declare const SolidACLAuthorizationSchema: Constructor<    {
id: string;
agents: any[];
agentClasses: any[];
agentGroups: any[];
accessTo: any[];
accessToClasses: any[];
default: any[];
modes: any[];
}> & Constructor<SolidModel> & typeof SolidModel;

export declare class SolidACLAuthorizationsRelation<Parent extends SolidModel = SolidModel> extends MultiModelRelation<Parent, SolidACLAuthorization, typeof SolidACLAuthorization> {
    aclUrl?: string;
    effectiveACLUrl?: string;
    constructor(parent: Parent);
    requireACLUrl(): string;
    isDocumentACL(): boolean | null;
    setForeignAttributes(): void;
    load(): Promise<SolidACLAuthorization[]>;
}

export declare interface SolidBelongsToManyRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidMultiModelDocumentRelation<Parent, Related, RelatedClass>, SolidBelongsToRelation {
}

export declare class SolidBelongsToManyRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidBelongsToManyRelationBase<Parent, Related, RelatedClass> implements ISolidDocumentRelation<Related>, BeforeParentCreateRelation, SynchronizesRelatedModels {
    load(): Promise<Related[]>;
    associate(foreignKey: ModelKey | unknown): void;
    disassociate(foreignKey: ModelKey | unknown): void;
    reset(related?: Related[]): void;
    clone(options?: RelationCloneOptions): this;
    __beforeParentCreate(): void;
    __synchronizeRelated(other: this): Promise<void>;
}

export declare const SolidBelongsToManyRelationBase: typeof BelongsToManyRelation;

export declare interface SolidBelongsToOneRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidSingleModelDocumentRelation<Parent, Related, RelatedClass>, SolidBelongsToRelation {
}

export declare class SolidBelongsToOneRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidBelongsToOneRelationBase<Parent, Related, RelatedClass> implements ISolidDocumentRelation<Related>, BeforeParentCreateRelation, SynchronizesRelatedModels {
    load(): Promise<Related | null>;
    reset(related?: Related[]): void;
    clone(options?: RelationCloneOptions): this;
    __beforeParentCreate(): void;
    __synchronizeRelated(other: this): Promise<void>;
}

export declare const SolidBelongsToOneRelationBase: typeof BelongsToOneRelation;

export declare class SolidBelongsToRelation {
    protected get protectedSolidBelongsTo(): ProtectedSolidBelongsToRelation;
    __loadDocumentModels(this: SolidBelongsToRelationThis, documentUrl: string, document: JsonLDGraph): Promise<void>;
}

export declare type SolidBelongsToRelationThis<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> = SolidBelongsToRelation & Relation<Parent, Related, RelatedClass>;

export declare type SolidBootedFieldDefinition = BootedFieldDefinition<{
    rdfProperty: string;
    rdfPropertyAliases: string[];
}>;

export declare type SolidBootedFieldsDefinition = BootedFieldsDefinition<{
    rdfProperty: string;
    rdfPropertyAliases: string[];
}>;

export declare class SolidContainer extends SolidContainerSchema {
    static boot(name?: string): void;
    static fromTypeIndex<T extends SolidContainer>(this: SolidModelConstructor<T>, typeIndexUrl: string, childrenModelClass: typeof SolidModel): Promise<T[]>;
    documents: SolidDocument[];
    relatedDocuments: SolidContainerDocumentsRelation;
    documentsRelationship(): Relation;
    register(typeIndex: string | SolidTypeIndex, childrenModelClasses: typeof SolidModel | Array<typeof SolidModel>): Promise<void>;
    static(): SolidContainerConstructor<this>;
    static<T extends keyof SolidContainerConstructor<this>>(property: T): SolidContainerConstructor<this>[T];
    protected contains<T extends typeof SolidModel>(model: T): SolidContainsRelation;
    protected newUrl(): string;
    protected newUniqueUrl(url?: string): string;
}

export declare type SolidContainerConstructor<T extends SolidContainer = SolidContainer> = SolidModelConstructor<T> & typeof SolidContainer;

export declare class SolidContainerDocumentsRelation extends SolidBelongsToManyRelation<SolidContainer, SolidDocument, typeof SolidDocument> {
    constructor(container: SolidContainer);
    get container(): SolidContainer;
    load(): Promise<SolidDocument[]>;
    __loadDocumentModels(documentUrl: string, document: JsonLDGraph): Promise<void>;
    private getLatestUpdateDate;
}

export declare const SolidContainerSchema: Constructor<    {
id: string;
name?: string | undefined;
createdAt?: Date | undefined;
updatedAt?: Date | undefined;
description?: string | undefined;
resourceUrls: any[];
}> & Constructor<SolidModel> & typeof SolidModel;

export declare class SolidContainsRelation<Parent extends SolidContainer = SolidContainer, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends BelongsToManyRelation<Parent, Related, RelatedClass> {
    constructor(parent: Parent, relatedClass: RelatedClass);
    setForeignAttributes(related: Related): void;
    load(): Promise<Related[]>;
    create(attributes?: Attributes): Promise<Related>;
    save(model: Related): Promise<Related>;
}

export declare class SolidDocument extends SolidDocumentSchema {
    static fromTypeIndex<T extends SolidDocument>(this: SolidModelConstructor<T>, typeIndexUrl: string, modelClass: typeof SolidModel): Promise<T[]>;
    register(typeIndex: string | SolidTypeIndex, modelClass: typeof SolidModel): Promise<void>;
}

export declare abstract class SolidDocumentRelation<Related extends SolidModel> {
    useSameDocument: boolean;
    protected documentModelsLoaded: boolean;
    usingSameDocument(useSameDocument?: boolean): this;
    protected abstract loadDocumentModels(modelsInSameDocument: Related[], modelsInOtherDocumentIds: string[]): void;
}

export declare type SolidDocumentRelationInstance<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> = Relation<Parent, Related, RelatedClass> & SolidDocumentRelation<Related> & ISolidDocumentRelation<Related>;

export declare const SolidDocumentSchema: Constructor<    {
id: string;
updatedAt?: Date | undefined;
}> & Constructor<SolidModel> & typeof SolidModel;

export declare class SolidEngine implements Engine {
    private config;
    private helper;
    private client;
    private cache;
    private listeners;
    constructor(fetch?: Fetch, config?: Partial<SolidEngineConfig>);
    setConfig(config: Partial<SolidEngineConfig>): void;
    getFetch(): Fetch;
    create(collection: string, document: EngineDocument, id?: string): Promise<string>;
    readOne(_: string, id: string): Promise<EngineDocument>;
    readMany(collection: string, filters?: EngineFilters): Promise<EngineDocumentsCollection>;
    update(collection: string, id: string, updates: EngineUpdates): Promise<void>;
    delete(collection: string, id: string): Promise<void>;
    addListener(listener: SolidEngineListener): () => void;
    removeListener(listener: SolidEngineListener): void;
    clearCache(): void;
    private getDocument;
    private emit;
    private getDocumentsForFilters;
    private getDocumentsFromUrls;
    private convertToEngineDocument;
    private validateJsonLDGraph;
    private extractJsonLDGraphUpdate;
    private extractJsonLDGraphItemsUpdate;
    private getUpdatePropertyOperationProperty;
    private extractJsonLDGraphItemPush;
    private extractJsonLDGraphTypes;
    private getJsonLDGraphProperties;
    private isJsonLDGraphUpdate;
    private isJsonLDGraphTypesFilter;
}

export declare interface SolidEngineConfig {
    useGlobbing: boolean;
    globbingBatchSize: number | null;
    concurrentFetchBatchSize: number | null;
    cachesDocuments: boolean;
}

export declare interface SolidEngineListener {
    onRDFDocumentLoaded?(url: string, metadata: RDFDocumentMetadata): void;
}

export declare type SolidFieldDefinition = FieldDefinition<{
    rdfProperty?: string;
    rdfPropertyAliases?: string[];
}>;

export declare type SolidFieldsDefinition = FieldsDefinition<{
    rdfProperty?: string;
    rdfPropertyAliases?: string[];
}>;

export declare interface SolidHasManyRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidMultiModelDocumentRelation<Parent, Related, RelatedClass>, SolidHasRelation {
}

export declare class SolidHasManyRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidHasManyRelationBase<Parent, Related, RelatedClass> implements ISolidDocumentRelation<Related>, BeforeParentCreateRelation {
    load(): Promise<Related[]>;
    reset(related?: Related[]): void;
    clone(options?: RelationCloneOptions): this;
    __beforeParentCreate(): void;
    protected loadRelatedModels(documentIds: string[]): Promise<Related[]>;
}

export declare const SolidHasManyRelationBase: typeof HasManyRelation;

export declare interface SolidHasOneRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidSingleModelDocumentRelation<Parent, Related, RelatedClass>, SolidHasRelation {
}

export declare class SolidHasOneRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidHasOneRelationBase<Parent, Related, RelatedClass> implements ISolidDocumentRelation<Related>, BeforeParentCreateRelation {
    load(): Promise<Related | null>;
    reset(related?: Related[]): void;
    clone(options?: RelationCloneOptions): this;
    __beforeParentCreate(): void;
}

export declare const SolidHasOneRelationBase: typeof HasOneRelation;

export declare class SolidHasRelation {
    protected get protectedSolidHas(): ProtectedSolidHasRelation;
    __loadDocumentModels(this: SolidHasRelationThis, documentUrl: string, document: JsonLDGraph): Promise<void>;
}

export declare type SolidHasRelationThis<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> = SolidHasRelation & Relation<Parent, Related, RelatedClass>;

export declare class SolidIsContainedByRelation<Parent extends SolidModel = SolidModel, Related extends SolidContainer = SolidContainer, RelatedClass extends SolidContainerConstructor<Related> = SolidContainerConstructor<Related>> extends SingleModelRelation<Parent, Related, RelatedClass> {
    constructor(parent: Parent, relatedClass: RelatedClass);
    isEmpty(): false;
    setForeignAttributes(related: Related): void;
    load(): Promise<Related | null>;
}

export declare type SolidMagicAttributes<S extends SolidSchemaDefinition, F extends SolidFieldsDefinition = GetFieldsDefinition<S>> = Pretty<MagicAttributes<S> & MagicAttributeProperties<Pick<F, GetArrayFields<F>>>>;

export declare class SolidModel extends SolidModelBase {
    static primaryKey: string;
    static fields: SolidFieldsDefinition;
    static classFields: string[];
    static rdfContext?: string;
    static rdfContexts: Record<string, string>;
    static rdfsClass?: string;
    static rdfsClasses: string[];
    static rdfsClassesAliases: string[][];
    static reservedRelations: string[];
    static defaultResourceHash: string;
    static slugField?: string;
    static mintsUrls: boolean;
    static history: boolean;
    static tombstone: boolean;
    protected static rdfPropertyFields?: Record<string, string>;
    protected static historyDisabled: WeakMap<SolidModel, void>;
    static getFieldDefinition(field: string, value?: unknown): SolidBootedFieldDefinition;
    static getFieldRdfProperty(field: string): string | null;
    static getRdfPropertyField(rdfProperty: string): string | null;
    static requireFetch(): Fetch_2;
    static requireFieldRdfProperty(field: string): string;
    static from<T extends SolidModel>(this: SolidModelConstructor<T>, parentUrl: string): SolidModelConstructor<T>;
    static at<T extends SolidModel>(this: SolidModelConstructor<T>, parentUrl: string): SolidModelConstructor<T>;
    static boot(name?: string): void;
    static aliasRdfPrefixes(aliases: Record<string, string>): void;
    static resetRdfAliases(): void;
    static replaceRdfPrefixes(replacements: Record<string, string>): void;
    static createFromEngineDocument<T extends SolidModel>(this: SolidModelConstructor<T>, id: Key, document: EngineDocument, resourceId?: string): Promise<T>;
    static createFromEngineDocument<T extends Model>(this: ModelConstructor<T>, id: Key, document: EngineDocument): Promise<T>;
    static find<T extends Model>(this: ModelConstructor<T>, id: Key): Promise<T | null>;
    static find<T extends SolidModel>(this: SolidModelConstructor<T>, id: Key): Promise<T | null>;
    static all<T extends Model>(this: ModelConstructor<T>, filters?: EngineFilters): Promise<T[]>;
    static all<T extends SolidModel>(this: SolidModelConstructor<T>, filters?: EngineFilters): Promise<T[]>;
    static prepareEngineFilters(filters?: EngineFilters): EngineFilters;
    static rdfTerm(property: string): string;
    static newFromJsonLD<T extends SolidModel>(this: SolidModelConstructor<T>, sourceJsonLD: JsonLD, baseUrl?: string, sourceResourceId?: string): Promise<T>;
    static createFromJsonLD<T extends SolidModel>(this: SolidModelConstructor<T>, jsonld: JsonLD, baseUrl?: string, resourceId?: string): Promise<T>;
    static createInDocument<T extends SolidModel>(this: SolidModelConstructor<T>, attributes: Attributes, documentUrl: string, resourceHash?: string): Promise<T>;
    static synchronize<T extends SolidModel>(this: SolidModelConstructor<T>, a: T, b: T): Promise<void>;
    static findMatchingResourceIds(quads: Quad[], baseUrl?: string): string[];
    protected static bootRdfContexts(initialClass?: typeof SolidModel, initialRdfContexts?: Record<string, string>): Record<string, string>;
    protected static bootRdfsClasses(initialClass?: typeof SolidModel): string[];
    protected static bootFields(): SolidBootedFieldsDefinition;
    protected static getDefaultRdfContext(): string;
    protected static withCollection<Result>(collection?: string | undefined | (() => Result | Promise<Result>), operation?: () => Result | Promise<Result>): Promise<Result>;
    url: string;
    deletedAt?: Date;
    authorizations?: SolidACLAuthorization[];
    metadata: Metadata;
    operations: Operation[];
    tombstone?: Tombstone;
    relatedAuthorizations: SolidACLAuthorizationsRelation<this>;
    relatedMetadata: SolidHasOneRelation<this, Metadata, SolidModelConstructor<Metadata>>;
    relatedOperations: OperationsRelation<this>;
    relatedTombstone: TombstoneRelation<this>;
    protected _documentExists: boolean;
    protected _sourceDocumentUrl: string | null;
    protected _trackedDirtyAttributes: Attributes;
    protected _removedResourceUrls: string[];
    protected _usesRdfAliases: boolean;
    protected _relations: Record<string, SolidRelation>;
    private _sourceSubject;
    private _history?;
    private _tombstone?;
    protected initialize(attributes: Attributes, exists: boolean): void;
    protected initializeRelations(): void;
    protected initializeRelationsEnabling(): void;
    protected initializeMetadataRelation(): void;
    static(): SolidModelConstructor<this>;
    static(property: 'fields'): SolidBootedFieldsDefinition;
    static(property: 'timestamps'): TimestampFieldValue[];
    static<T extends keyof SolidModelConstructor<this>>(property: T): SolidModelConstructor<this>[T];
    save(collection?: string): Promise<this>;
    saveInDocument(documentUrl: string, resourceHash?: string): Promise<this>;
    delete(): Promise<this>;
    softDelete(): Promise<this>;
    registerInTypeIndex(typeIndexUrl: string): Promise<void>;
    mintUrl(documentUrl?: string, documentExists?: boolean, resourceHash?: string): void;
    toJsonLD(options?: SolidModelSerializationOptions): Record<string, unknown>;
    toTurtle(options?: SolidModelSerializationOptions): Promise<string>;
    getIdAttribute(): string;
    setExists(exists: boolean): void;
    documentExists(): boolean;
    setDocumentExists(documentExists: boolean): void;
    isDirty(field?: string, ignoreRelations?: boolean): boolean;
    isSoftDeleted(): boolean;
    enableHistory(): void;
    disableHistory(): void;
    disableTombstone(): void;
    enableTombstone(): void;
    leavesTombstone(): boolean;
    cleanDirty(ignoreRelations?: boolean): void;
    fixMalformedAttributes(): void;
    tracksHistory(): boolean;
    withoutTrackingHistory<T>(operation: () => T): T;
    withoutTrackingHistory<T>(operation: () => Promise<T>): Promise<T>;
    getHistoryHash(): string | null;
    rebuildAttributesFromHistory(): void;
    getDocumentUrl(): string | null;
    requireDocumentUrl(): string;
    usesRdfAliases(): boolean;
    getSourceDocumentUrl(): string | null;
    setSourceDocumentUrl(sourceDocumentUrl: string | null): void;
    requireContainerUrl(): string;
    getContainerUrl(): string | null;
    getSourceContainerUrl(): string | null;
    setAttribute(field: string, value: unknown): void;
    unsetAttribute(field: string): void;
    setCreatedAtAttribute(value: unknown): void;
    setUpdatedAtAttribute(value: unknown): void;
    getCreatedAtAttribute(): Date;
    getUpdatedAtAttribute(): Date;
    getDeletedAtAttribute(): Date | undefined;
    getDocumentModels(_documentModels?: Set<SolidModel>): SolidModel[];
    getRemovedDocumentModels(): SolidModel[];
    getDirtyDocumentModels(): SolidModel[];
    requireRelation<T extends SolidRelation = SolidRelation>(relation: string): T;
    requireRelation<T extends Relation = Relation>(relation: string): T;
    getRelationModel<T extends SolidModel>(relation: string): T | null;
    getRelationModel<T extends Model>(relation: string): T | null;
    getRelationModels<T extends SolidModel>(relation: string): T[] | null;
    getRelationModels<T extends Model>(relation: string): T[] | null;
    setRelationModel(relation: string, model: SolidModel | null): void;
    setRelationModels(relation: string, models: SolidModel[] | null): void;
    getRelatedModels(_relatedModels?: Set<SolidModel>): SolidModel[];
    metadataRelationship(): Relation;
    operationsRelationship(): Relation;
    tombstoneRelationship(): Relation;
    authorizationsRelationship(): Relation;
    onMoved(newResourceUrl: string, newDocumentUrl?: string): void;
    protected getDefaultCollection(): string;
    protected createFromEngineDocument(id: Key, document: EngineDocument, resourceId?: string): Promise<this>;
    protected createManyFromEngineDocuments(documents: Record<string, EngineDocument>): Promise<this[]>;
    protected loadDocumentModels(documentUrl: string, document: EngineDocument): Promise<void>;
    protected beforeSave(ignoreRelations?: boolean): Promise<void>;
    protected beforeDocumentSave(): Promise<void>;
    protected beforeCreate(): Promise<void>;
    protected beforeUpdate(): Promise<void>;
    protected performSave(): Promise<void>;
    protected performDelete(): Promise<void>;
    protected afterSave(ignoreRelations?: boolean): Promise<void>;
    protected onPrimaryKeyUpdated(value: Key | null, oldValue: Key | null): void;
    protected syncDirty(): Promise<string>;
    protected deleteModelsFromEngine(models: this[]): Promise<void>;
    protected addDirtyHistoryOperations(): Promise<void>;
    protected addHistoryOperations(operations: Operation[]): void;
    protected removeDuplicatedHistoryOperations(): void;
    protected hasOne<T extends typeof SolidModel>(relatedClass: T, foreignKeyField?: string, localKeyField?: string): SolidHasOneRelation;
    protected hasOne<T extends typeof Model>(relatedClass: T, foreignKeyField?: string, localKeyField?: string): Relation;
    protected hasMany<T extends typeof SolidModel>(relatedClass: T, foreignKeyField?: string, localKeyField?: string): SolidHasManyRelation;
    protected hasMany<T extends typeof Model>(relatedClass: T, foreignKeyField?: string, localKeyField?: string): Relation;
    protected belongsToOne<T extends typeof SolidModel>(relatedClass: T, foreignKeyField?: string, localKeyField?: string): SolidBelongsToOneRelation;
    protected belongsToOne<T extends typeof Model>(relatedClass: T, foreignKeyField?: string, localKeyField?: string): Relation;
    protected belongsToMany<T extends typeof SolidModel>(relatedClass: T, foreignKeyField?: string, localKeyField?: string): SolidBelongsToManyRelation;
    protected belongsToMany<T extends typeof Model>(relatedClass: T, foreignKeyField?: string, localKeyField?: string): Relation;
    protected isContainedBy<T extends typeof SolidContainer>(model: T): SolidIsContainedByRelation;
    protected documentContainsMany<T extends typeof SolidModel>(model: T): DocumentContainsManyRelation;
    protected toEngineDocument(): EngineDocument;
    protected getDirtyEngineDocumentUpdates(ignoreRelations?: boolean): EngineUpdates;
    protected parseEngineDocumentAttributes(id: Key, document: EngineDocument, resourceId?: string): Promise<Attributes>;
    protected castAttribute(value: unknown, options?: ModelCastAttributeOptions): unknown;
    protected newUrl(documentUrl?: string, resourceHash?: string): string;
    protected newUrlDocumentUrl(): string;
    protected newUrlDocumentUrlSlug(): string | null;
    protected newUrlResourceHash(): string;
    protected newUniqueUrl(url?: string): string;
    protected guessCollection(): string | undefined;
    protected mintDocumentModelsKeys(models: SolidModel[]): void;
    private getOperationValue;
    private addArrayHistoryOperations;
    private reconcileModelTimestamps;
}

export declare const SolidModelBase: typeof Model & Cr<DeletesModels & SerializesToJsonLD & ManagesPermissions>;

export declare type SolidModelConstructor<T extends SolidModel = SolidModel> = Constructor<T> & typeof SolidModel;

export declare interface SolidModelSerializationOptions {
    ids?: boolean;
    timestamps?: boolean;
    history?: boolean;
}

export declare class SolidMultiModelDocumentRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidDocumentRelation<Related> {
    __newModels: Related[];
    __removedDocumentModels: Related[];
    __modelsInSameDocument?: Related[];
    __modelsInOtherDocumentIds?: string[];
    protected get protectedSolidMulti(): ProtectedSolidMultiRelation;
    private get super();
    isEmpty(this: SolidMultiModelDocumentRelationThis): boolean | null;
    /**
     * This method will create an instance of the related model and call the [[save]] method.
     *
     * @param attributes Attributes to create the related instance.
     */
    create(this: SolidMultiModelDocumentRelationThis<Parent, Related, RelatedClass>, attributes?: Attributes): Promise<Related>;
    /**
     * This method will bind up all the relevant data (foreignKey, inverse relations, etc.) and save the model.
     * If the parent model does not exist and both models will be stored in the same document, the model will
     * be saved when the parent model is saved instead.
     *
     * @param model Related model instance to save.
     */
    save(this: SolidMultiModelDocumentRelationThis, model: Related): Promise<Related>;
    remove(this: SolidMultiModelDocumentRelationThis<Parent, Related, RelatedClass>, keyOrModel: Key | Related): Promise<void>;
    detach(this: SolidMultiModelDocumentRelationThis<Parent, Related, RelatedClass>, keyOrModel: string | Related): void;
    addRelated(related: Related): void;
    isRelated(model: Related): boolean;
    protected cloneSolidData(clone: SolidMultiModelDocumentRelationThis<Parent, Related, RelatedClass>): void;
    protected loadDocumentModels(this: SolidMultiModelDocumentRelationThis<Parent, Related, RelatedClass>, modelsInSameDocument: Related[], modelsInOtherDocumentIds: string[]): void;
}

export declare type SolidMultiModelDocumentRelationInstance<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> = SolidMultiModelDocumentRelationThis<Parent, Related, RelatedClass>;

export declare type SolidMultiModelDocumentRelationThis<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> = SolidMultiModelDocumentRelation<Parent, Related, RelatedClass> & MultiModelRelation<Parent, Related, RelatedClass>;

export declare type SolidRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> = Relation<Parent, Related, RelatedClass>;

export declare type SolidSchemaDefinition = SchemaDefinition<{
    rdfProperty?: string;
    rdfPropertyAliases?: string[];
}> & Partial<{
    rdfContext: string;
    rdfContexts: Record<string, string>;
    rdfsClass: string;
    rdfsClasses: string[];
    rdfsClassesAliases: string[][];
    defaultResourceHash: string;
    slugField: string;
    history: boolean;
    tombstone: boolean;
}>;

export declare class SolidSingleModelDocumentRelation<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> extends SolidDocumentRelation<Related> {
    __newModel?: Related;
    __modelInSameDocument?: Related;
    __modelInOtherDocumentId?: string;
    private get super();
    isEmpty(this: SolidSingleModelDocumentRelationThis): boolean | null;
    /**
     * This method will create an instance of the related model and call the [[save]] method.
     *
     * @param attributes Attributes to create the related instance.
     */
    create(this: SolidSingleModelDocumentRelationThis<Parent, Related, RelatedClass>, attributes?: Attributes): Promise<Related>;
    /**
     * This method will bind up all the relevant data (foreignKey, inverse relations, etc.) and save the model.
     * If the parent model does not exist and both models will be stored in the same document, the model will
     * be saved when the parent model is saved instead.
     *
     * @param model Related model instance to save.
     */
    save(this: SolidSingleModelDocumentRelationThis, model: Related): Promise<Related>;
    addRelated(related: Related): void;
    isRelated(related: Related): boolean;
    protected cloneSolidData(clone: SolidSingleModelDocumentRelationThis<Parent, Related, RelatedClass>): void;
    protected loadDocumentModels(this: SolidSingleModelDocumentRelationThis<Parent, Related, RelatedClass>, modelsInSameDocument: Related[], modelsInOtherDocumentIds: string[]): void;
    private assertNotLoaded;
}

export declare type SolidSingleModelDocumentRelationInstance<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> = SolidSingleModelDocumentRelationThis<Parent, Related, RelatedClass>;

export declare type SolidSingleModelDocumentRelationThis<Parent extends SolidModel = SolidModel, Related extends SolidModel = SolidModel, RelatedClass extends SolidModelConstructor<Related> = SolidModelConstructor<Related>> = SolidSingleModelDocumentRelation<Parent, Related, RelatedClass> & SingleModelRelation<Parent, Related, RelatedClass>;

export declare class SolidTypeIndex extends SolidTypeIndexSchema {
    static createPublic<T extends SolidTypeIndex>(this: SolidModelConstructor<T>, user: SolidUserProfile): Promise<T>;
    static createPrivate<T extends SolidTypeIndex>(this: SolidModelConstructor<T>, user: SolidUserProfile): Promise<T>;
    registrations: SolidTypeRegistration[];
    relatedRegistrations: DocumentContainsManyRelation<this, SolidTypeRegistration, typeof SolidTypeRegistration>;
    registrationsRelationship(): Relation;
    findContainer<T extends SolidContainer = SolidContainer>(modelClass: SolidModelConstructor, containerClass?: SolidModelConstructor<T>): Promise<T | null>;
}

export declare const SolidTypeIndexSchema: Constructor<    {
id: string;
}> & Constructor<SolidModel> & typeof SolidModel;

export declare class SolidTypeRegistration extends SolidTypeRegistrationSchema {
}

export declare const SolidTypeRegistrationSchema: Constructor<    {
id: string;
instance?: any;
forClass: any[];
instanceContainer?: any;
}> & Constructor<SolidModel> & typeof SolidModel;

/**
 * @deprecated Use bootSolidModels() instead.
 */
export declare class SoukaiSolid {
    /**
     * @deprecated Use bootSolidModels() instead.
     */
    loadSolidModels(): void;
}

export declare interface SynchronizesRelatedModels extends Relation {
    __synchronizeRelated(other: Relation): Promise<void>;
}

export declare function synchronizesRelatedModels(relation: Relation): relation is SynchronizesRelatedModels;

export declare class Tombstone extends Metadata {
    static rdfsClass: string;
}

export declare class TombstoneRelation<Parent extends SolidModel = SolidModel> extends SingleModelRelation<Parent, Tombstone, typeof Tombstone> {
    constructor(parent: Parent);
    create(): void;
    load(): Promise<Tombstone | null>;
    setForeignAttributes(): void;
}

export declare type TypedFetch = (input: RequestInfo, options?: RequestInit) => Promise<Response>;

export declare class UnsetPropertyOperation extends UnsetPropertyOperationSchema {
}

export declare const UnsetPropertyOperationSchema: Constructor<    {
id: string;
createdAt: Date;
updatedAt: Date;
}> & Constructor<PropertyOperation> & typeof SolidModel;

export { }
